// Auctioneer.java
// Fall 2004, COP4331: Online auction example application
// 


/**
   the auctioneer class.

   NOTE: The purpose of this code is to illustrate implementation starting 
   from UML class and sequence diagrams. It may be incomplete and may not 
   compile.
*/
public class Auctioneer {
    /**
       constructs the Auctioneer. Saves the active list, listing and archive objects.
    */
    public Auctioneer(ActiveList activeList, AuctionListing al, ListingArchive arch) {
	this.activeList = activeList;
	this.al = al;
	this.arch = arch;
    }

    /**
       complete an auction. This method is invoked asynchronously by a timer 
       object when an auction must be completed.
       Derived from the "Complete Auction" use case.
     */
    public void auctionCompleted() {
	// remove listing from active list:
	this.activeList.removeListing(this.al);

	// add listing to the archive:
	this.arch.addListing(this.al);

	// get the highest bid:
	Bid bid = this.al.getHighestBid();
	// the "Complete Auction" use case only shows the success scenario.
	// an alternate scenario would describe the case when an auction has
	// no bids. In this case, getHighestBid() would return null
	
	// signal seller. If bid==null, then there was nol bid for this item:
	this.al.getSeller().auctionCompleted(this.al.getItem(), bid);
	    
	// if at least a valid bid, signal buyer, too:
	if (bid != null) {
	
	    // signal the buyer. The Buyer will call seller.buyItem(item, bid):
	    bid.getBuyer().auctionCompleted(this.al.getItem(), bid);
	}
	else {
	    // bid == null. No bid for this auction (item).
	    // nothing to do.
	}
    }


    /**
       enter a new bid for an item. 
       Derived from the "Bid for Item" use case.
       @param bid the new bid for this auction.
    */
    public void enterBid(Bid bid) {
	// first, validate the bid. validateBid() will compare with the highest bid
	// and return TRUE if new bid is correct (new price > previous highest price):
	if (this.validateBid(bid)) {

	    // tell the listing object to save this bid object.
	    // The AuctionListing object will add it to the bidding history:
	    this.al.addBid(bid);
	}
    }


    /**
       validate a new bid.
       Derived from the "Bid For Item" use case.
       @param bid the new bid for this auction.
    */
    public boolean validateBid(Bid bid) {
	boolean isValid = true;
	// get currently highest bid:
	Bid highestBid = this.al.getHighestBid();

	// getHighestBid returns null if there is no bid:
	if (highestBid == null) {
	    isValid = false;
	}
	else {
	    // compare prices:
	    if (bid.getPrice() <= highestBid.getPrice()) {
		isValid = false;
	    }
	}
	
	return isValid;
    }

    /**
       schedule auction completion. 
       Derived from the "List Item" use case.
    */
    public void scheduleCompletion() {
	// create new Timer object:
	java.util.Timer timer = new Timer();
	
	try {
	    // schedule timer to expire when auction will end:
	    timer.schedule(new AuctionCompletedTask(this), al.getDeadline());

	    // when timer expires, the JVM calls AuctionCompletedTask.run()
	    // and this will call auctioneer.auctionCompleted()
	} catch (Exception ex) {
	    // ... indicate this error has occured ...
	    throw(ex);
	}
    }

    /**
       inner class that extends a TimerTask to provide a callback (run()) that
       invokes the auctioneer's auctionCompleted() method.
       Auctioneer operation.
       When the timer expires, the JVM invokes the run() method.
       Derived from the "Complete Auction" use case.
    */
    private class AuctionCompletedTask extends java.util.TimerTask {
	/**
	   constructs the task. Save the auctioneer object.
	   @param auctioneer the Auctioneer object.
	   @see Auctioneer
	*/
	AuctionCompletedTask(Auctioneer auctioneer) {
	    this.auctioneer = auctioneer;
	}

	/**
	   the action to be performed by this task when timer expires.
	   @see Auctioneer
	*/
	public void run() {
	    // call the auctioneer object:
	    this.auctioneer.auctionPerformed();
	}
    } // AuctionCompletedTask class


    
    // Auctioneer instance variables:
    /**
       the auction listing describing the item, bids and other elements
    */
    private AuctionListing al;

    /**
       auction listing archive
    */
    private ListingArchive arch;
};
